// midpoint method
// load json

Point[] points;
Link[] links;
long startTime = 0;
long lastUpdatedTime = 0;
MidpointOdeSolver ode;

void setup() {
  size(640, 360);
  
  loadGeometry();
  
  ode = new MidpointOdeSolver(points.length);
  
  startTime = millis();
  lastUpdatedTime = millis();
}

void loadGeometry() {
  JSONObject json = loadJSONObject("data.json");
  println(json);
  
  JSONArray jsonPoints = json.getJSONArray("points");
  points = new Point[jsonPoints.size()];
  for(int i = 0; i < jsonPoints.size(); i++) {
    try {
      JSONObject jsonPoint = jsonPoints.getJSONObject(i); 
      points[i] = new Point();
      points[i].p.x = jsonPoint.getFloat("x");
      points[i].p.y = jsonPoint.getFloat("y");
      points[i].m = jsonPoint.getFloat("m");
      points[i].fixed = jsonPoint.getBoolean("fixed");
    } catch(Exception e) {
      println("Exception while read points.");
      e.printStackTrace();
    }
  }
  
  JSONArray jsonLinks = json.getJSONArray("links");
  links = new Link[jsonLinks.size()];
  for(int i = 0; i < jsonLinks.size(); i++) {
    try {
      JSONObject jsonLink = jsonLinks.getJSONObject(i); 
      links[i] = new Link();
      links[i].p0 = jsonLink.getInt("p0");
      links[i].p1 = jsonLink.getInt("p1");
      links[i].k = jsonLink.getFloat("k");
      links[i].l = PVector.sub(points[links[i].p0].p, points[links[i].p1].p).mag();
    } catch(Exception e) {
      println("Exception while read links.");
      e.printStackTrace();
    }
  }
}

void draw() {
  background(0);

  long now = millis();
  float delta = (now - lastUpdatedTime) / 1000.f;
  lastUpdatedTime = now;
  
  // Wait a sec.
  if(now - startTime < 1000) return;
  
  ode.solve(points, links, delta);

  for(Point p : points) {
    p.draw();
  }

  stroke(127);
  for(Link l : links) {
    line(points[l.p1].p.x, points[l.p1].p.y, points[l.p0].p.x, points[l.p0].p.y);
  }

}
